/**
 * Copyright (c) 2017 Inria.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Inria - initial API and implementation
 */
package org.eclipse.gemoc.xdsmlframework.test.lib;

import com.google.common.base.Objects;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.function.Consumer;
import java.util.zip.ZipFile;
import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.expressions.IEvaluationContext;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfigurationType;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.ui.IDebugUIConstants;
import org.eclipse.gemoc.xdsmlframework.ide.ui.XDSMLFrameworkUI;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.osgi.container.Module;
import org.eclipse.osgi.container.ModuleRevision;
import org.eclipse.osgi.internal.framework.EquinoxBundle;
import org.eclipse.osgi.storage.BundleInfo;
import org.eclipse.osgi.storage.bundlefile.BundleFile;
import org.eclipse.pde.core.target.ITargetDefinition;
import org.eclipse.pde.core.target.ITargetLocation;
import org.eclipse.pde.core.target.ITargetPlatformService;
import org.eclipse.pde.core.target.LoadTargetDefinitionJob;
import org.eclipse.pde.internal.core.target.TargetPlatformService;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.ISources;
import org.eclipse.ui.IWindowListener;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.dialogs.IOverwriteQuery;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.internal.wizards.datatransfer.ZipLeveledStructureProvider;
import org.eclipse.ui.intro.IIntroManager;
import org.eclipse.ui.intro.IIntroPart;
import org.eclipse.ui.wizards.datatransfer.ImportOperation;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.junit4.ui.util.IResourcesSetupUtil;
import org.eclipse.xtext.junit4.ui.util.JavaProjectSetupUtil;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Conversions;
import org.eclipse.xtext.xbase.lib.Exceptions;
import org.eclipse.xtext.xbase.lib.ExclusiveRange;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.InputOutput;
import org.eclipse.xtext.xbase.lib.IntegerRange;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.StringExtensions;
import org.junit.Assert;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

/**
 * Class containing helper methods for testing a workspace in a GEMOC Language workbench
 */
@SuppressWarnings("all")
public class WorkspaceTestHelper {
  public final static String CMD_PROJECT_CLEAN = "org.eclipse.ui.project.cleanAction";
  
  /**
   * Value to use by default for SWTBotPreferences.TIMEOUT
   * It must be used for any GEMOC test using swtbot
   * we should increase this value up to the value where we don't have any false failed tests
   */
  public final static int SWTBotPreferencesTIMEOUT_4_GEMOC = 8000;
  
  public void init() {
    Display _default = Display.getDefault();
    _default.syncExec(new Runnable() {
      @Override
      public void run() {
        try {
          IWorkbench _workbench = PlatformUI.getWorkbench();
          IWorkbench _workbench_1 = PlatformUI.getWorkbench();
          IWorkbenchWindow _activeWorkbenchWindow = _workbench_1.getActiveWorkbenchWindow();
          _workbench.showPerspective(XDSMLFrameworkUI.ID_PERSPECTIVE, _activeWorkbenchWindow);
          WorkspaceTestHelper.this.closeWelcomePage();
        } catch (Throwable _e) {
          throw Exceptions.sneakyThrow(_e);
        }
      }
    });
  }
  
  public IProject getProject(final String projectName) {
    IWorkspace _workspace = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = _workspace.getRoot();
    return _root.getProject(projectName);
  }
  
  public boolean projectExists(final String projectName) {
    IProject _project = this.getProject(projectName);
    return _project.exists();
  }
  
  private IProject deployProjectResult = null;
  
  public IProject deployProject(final String projectName, final String zipLocation) {
    this.deployProjectResult = null;
    final ArrayList<Throwable> thrownException = CollectionLiterals.<Throwable>newArrayList();
    Display _default = Display.getDefault();
    final Runnable _function = new Runnable() {
      @Override
      public void run() {
        try {
          final IProject newProject = JavaProjectSetupUtil.createSimpleProject(projectName);
          final ZipFile zip = new ZipFile(zipLocation);
          final ZipLeveledStructureProvider structureProvider = new ZipLeveledStructureProvider(zip);
          final IOverwriteQuery queryOverwrite = new IOverwriteQuery() {
            @Override
            public String queryOverwrite(final String file) {
              return IOverwriteQuery.ALL;
            }
          };
          IProject _project = newProject.getProject();
          IPath _fullPath = _project.getFullPath();
          Object _root = structureProvider.getRoot();
          ImportOperation _importOperation = new ImportOperation(_fullPath, _root, structureProvider, queryOverwrite);
          NullProgressMonitor _nullProgressMonitor = new NullProgressMonitor();
          _importOperation.run(_nullProgressMonitor);
          zip.close();
          IProject _project_1 = newProject.getProject();
          WorkspaceTestHelper.this.deployProjectResult = _project_1;
        } catch (final Throwable _t) {
          if (_t instanceof Exception) {
            final Exception e = (Exception)_t;
            thrownException.add(e);
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
      }
    };
    _default.syncExec(_function);
    final Function1<Throwable, Boolean> _function_1 = new Function1<Throwable, Boolean>() {
      @Override
      public Boolean apply(final Throwable e) {
        try {
          throw new Exception(e);
        } catch (Throwable _e) {
          throw Exceptions.sneakyThrow(_e);
        }
      }
    };
    IterableExtensions.<Throwable>forall(thrownException, _function_1);
    return this.deployProjectResult;
  }
  
  public void closeWelcomePage() {
    IWorkbench _workbench = PlatformUI.getWorkbench();
    IIntroManager _introManager = _workbench.getIntroManager();
    IIntroPart _intro = _introManager.getIntro();
    boolean _tripleNotEquals = (_intro != null);
    if (_tripleNotEquals) {
      IWorkbench _workbench_1 = PlatformUI.getWorkbench();
      IIntroManager _introManager_1 = _workbench_1.getIntroManager();
      IWorkbench _workbench_2 = PlatformUI.getWorkbench();
      IIntroManager _introManager_2 = _workbench_2.getIntroManager();
      IIntroPart _intro_1 = _introManager_2.getIntro();
      _introManager_1.closeIntro(_intro_1);
    }
  }
  
  public void assertNoMarkers() {
    IWorkspace _workspace = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = _workspace.getRoot();
    IProject[] _projects = _root.getProjects();
    final Consumer<IProject> _function = new Consumer<IProject>() {
      @Override
      public void accept(final IProject project) {
        try {
          IMarker[] _findMarkers = project.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
          final Consumer<IMarker> _function = new Consumer<IMarker>() {
            @Override
            public void accept(final IMarker m) {
              try {
                StringConcatenation _builder = new StringConcatenation();
                _builder.append("Found marker ");
                Object _attribute = m.getAttribute(IMarker.MESSAGE);
                _builder.append(_attribute, "");
                _builder.append(" (");
                Object _attribute_1 = m.getAttribute(IMarker.SEVERITY);
                _builder.append(_attribute_1, "");
                _builder.append(")");
                InputOutput.<String>println(_builder.toString());
                Object _attribute_2 = m.getAttribute(IMarker.MESSAGE);
                String _plus = ("Unexpected marker: " + _attribute_2);
                String _plus_1 = (_plus + " on ");
                IResource _resource = m.getResource();
                IPath _fullPath = _resource.getFullPath();
                String _plus_2 = (_plus_1 + _fullPath);
                Object _attribute_3 = m.getAttribute(IMarker.SEVERITY);
                boolean _equals = Objects.equal(_attribute_3, Integer.valueOf(IMarker.SEVERITY_ERROR));
                Assert.assertFalse(_plus_2, _equals);
              } catch (Throwable _e) {
                throw Exceptions.sneakyThrow(_e);
              }
            }
          };
          ((List<IMarker>)Conversions.doWrapArray(_findMarkers)).forEach(_function);
        } catch (Throwable _e) {
          throw Exceptions.sneakyThrow(_e);
        }
      }
    };
    ((List<IProject>)Conversions.doWrapArray(_projects)).forEach(_function);
  }
  
  public void assertNoMarkers(final String filename) {
    try {
      IWorkspace _workspace = ResourcesPlugin.getWorkspace();
      IWorkspaceRoot _root = _workspace.getRoot();
      Path _path = new Path(filename);
      final IFile mlgFile = _root.getFile(_path);
      IMarker[] _findMarkers = mlgFile.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
      final Consumer<IMarker> _function = new Consumer<IMarker>() {
        @Override
        public void accept(final IMarker it) {
          try {
            StringConcatenation _builder = new StringConcatenation();
            _builder.append("Found marker ");
            Object _attribute = it.getAttribute(IMarker.MESSAGE);
            _builder.append(_attribute, "");
            _builder.append(" (");
            Object _attribute_1 = it.getAttribute(IMarker.SEVERITY);
            _builder.append(_attribute_1, "");
            _builder.append(")");
            InputOutput.<String>println(_builder.toString());
            Object _attribute_2 = it.getAttribute(IMarker.MESSAGE);
            String _plus = ("Unexpected marker: " + _attribute_2);
            String _plus_1 = (_plus + " on file ");
            String _plus_2 = (_plus_1 + filename);
            Object _attribute_3 = it.getAttribute(IMarker.SEVERITY);
            boolean _equals = Objects.equal(_attribute_3, Integer.valueOf(IMarker.SEVERITY_ERROR));
            Assert.assertFalse(_plus_2, _equals);
          } catch (Throwable _e) {
            throw Exceptions.sneakyThrow(_e);
          }
        }
      };
      ((List<IMarker>)Conversions.doWrapArray(_findMarkers)).forEach(_function);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  /**
   * Usage : projectName/folder/file
   */
  public void assertFileExists(final String filename) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = ws.getRoot();
    Path _path = new Path(filename);
    IFile _file = _root.getFile(_path);
    boolean _exists = _file.exists();
    Assert.assertTrue(
      ("Cannot find file " + filename), _exists);
  }
  
  /**
   * Usage : projectName/folder/file
   */
  public void assertFileDontExists(final String filename) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = ws.getRoot();
    Path _path = new Path(filename);
    IFile _file = _root.getFile(_path);
    boolean _exists = _file.exists();
    Assert.assertFalse(
      ("Found file " + filename), _exists);
  }
  
  /**
   * Usage : projectName/folder
   */
  public void assertFolderExists(final String foldername) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = ws.getRoot();
    Path _path = new Path(foldername);
    IFolder _folder = _root.getFolder(_path);
    boolean _exists = _folder.exists();
    Assert.assertTrue(
      ("Cannot find file " + foldername), _exists);
  }
  
  /**
   * Check if {@link project} exist
   */
  public void assertProjectExists(final String project) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = ws.getRoot();
    IProject _project = _root.getProject(project);
    boolean _exists = _project.exists();
    Assert.assertTrue(
      ("Cannot find project " + project), _exists);
  }
  
  /**
   * Check if {@link project} doesn't exist
   */
  public void assertProjectDoesntExists(final String project) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = ws.getRoot();
    IProject _project = _root.getProject(project);
    boolean _exists = _project.exists();
    Assert.assertFalse(
      ("Can find project " + project), _exists);
  }
  
  /**
   * search in the given list if it contains the searchedString, report the error with a message
   */
  public void assertContains(final String baseMessage, final String searchedString, final List<String> list) {
    boolean _contains = list.contains(searchedString);
    Assert.assertTrue(((baseMessage + " ") + searchedString), _contains);
  }
  
  /**
   * Creates and lauches a new run configuration for {@link project}
   * using {@link mainClass} as the main Java class.
   * 
   * @return the produced console output
   */
  public String runMainClass(final IProject project, final String mainClass) {
    try {
      final String outputFileName = "output.txt";
      DebugPlugin _default = DebugPlugin.getDefault();
      final ILaunchManager manager = _default.getLaunchManager();
      final ILaunchConfigurationType type = manager.getLaunchConfigurationType(IJavaLaunchConfigurationConstants.ID_JAVA_APPLICATION);
      final ILaunchConfigurationWorkingCopy newLaunchConfig = type.newInstance(project, "RunMainTransfo");
      String _name = project.getName();
      newLaunchConfig.setAttribute(IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, _name);
      newLaunchConfig.setAttribute(IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME, mainClass);
      StringConcatenation _builder = new StringConcatenation();
      _builder.append("${workspace_loc:/");
      String _name_1 = project.getName();
      _builder.append(_name_1, "");
      _builder.append("/");
      _builder.append(outputFileName, "");
      _builder.append("}");
      newLaunchConfig.setAttribute(IDebugUIConstants.ATTR_CAPTURE_IN_FILE, _builder.toString());
      newLaunchConfig.doSave();
      final IFile outputFile = project.getFile(outputFileName);
      boolean _exists = outputFile.exists();
      boolean _not = (!_exists);
      if (_not) {
        byte[] _bytes = "".getBytes();
        ByteArrayInputStream _byteArrayInputStream = new ByteArrayInputStream(_bytes);
        outputFile.create(_byteArrayInputStream, true, null);
      }
      final ILaunch launch = newLaunchConfig.launch(ILaunchManager.RUN_MODE, null);
      final long TIMEOUT = ((1000 * 60) * 5);
      final long startTime = System.currentTimeMillis();
      while (((!launch.isTerminated()) && (System.currentTimeMillis() < (startTime + TIMEOUT)))) {
        try {
          Thread.sleep(50);
        } catch (final Throwable _t) {
          if (_t instanceof InterruptedException) {
            final InterruptedException e = (InterruptedException)_t;
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
      }
      outputFile.refreshLocal(IResource.DEPTH_ONE, null);
      final InputStream bi = outputFile.getContents();
      int _available = bi.available();
      final byte[] buffer = new byte[_available];
      bi.read(buffer);
      final String out = new String(buffer);
      return out;
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public void debug() {
    IWorkspace _workspace = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = _workspace.getRoot();
    IProject[] _projects = _root.getProjects();
    final Consumer<IProject> _function = new Consumer<IProject>() {
      @Override
      public void accept(final IProject project) {
        try {
          String _name = project.getName();
          String _plus = ("Error markers: " + _name);
          InputOutput.<String>println(_plus);
          IMarker[] _findMarkers = project.findMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
          final Consumer<IMarker> _function = new Consumer<IMarker>() {
            @Override
            public void accept(final IMarker marker) {
              try {
                IResource _resource = marker.getResource();
                String _name = _resource.getName();
                String _plus = ("   Resource: " + _name);
                InputOutput.<String>println(_plus);
                Object _attribute = marker.getAttribute(IMarker.LOCATION);
                String _plus_1 = ("   Location: " + _attribute);
                InputOutput.<String>println(_plus_1);
                Object _attribute_1 = marker.getAttribute(IMarker.MESSAGE);
                String _plus_2 = ("   Message: " + _attribute_1);
                String _plus_3 = (_plus_2 + "\n");
                InputOutput.<String>println(_plus_3);
              } catch (Throwable _e) {
                throw Exceptions.sneakyThrow(_e);
              }
            }
          };
          ((List<IMarker>)Conversions.doWrapArray(_findMarkers)).forEach(_function);
          InputOutput.<String>println("\n");
          String _name_1 = project.getName();
          String _plus_1 = ("Classpath: " + _name_1);
          InputOutput.<String>println(_plus_1);
          final IJavaProject jProject = JavaCore.create(project);
          final StringBuffer cp = new StringBuffer();
          IClasspathEntry[] _resolvedClasspath = jProject.getResolvedClasspath(false);
          final Consumer<IClasspathEntry> _function_1 = new Consumer<IClasspathEntry>() {
            @Override
            public void accept(final IClasspathEntry entry) {
              IPath _path = entry.getPath();
              String _string = _path.toString();
              String _plus = (" " + _string);
              String _plus_1 = (_plus + "\n");
              cp.append(_plus_1);
            }
          };
          ((List<IClasspathEntry>)Conversions.doWrapArray(_resolvedClasspath)).forEach(_function_1);
          final String str = cp.toString();
          InputOutput.<String>println(str);
        } catch (Throwable _e) {
          throw Exceptions.sneakyThrow(_e);
        }
      }
    };
    ((List<IProject>)Conversions.doWrapArray(_projects)).forEach(_function);
  }
  
  /**
   * Sets a target platform in the test platform to get workspace builds OK
   * with PDE
   * 
   * @throws Exception
   */
  public void setTargetPlatform() throws Exception {
    final ITargetPlatformService tpService = TargetPlatformService.getDefault();
    final ITargetDefinition targetDef = tpService.newTarget();
    targetDef.setName("Tycho platform");
    Bundle _bundle = Platform.getBundle("org.eclipse.core.runtime");
    BundleContext _bundleContext = _bundle.getBundleContext();
    final Bundle[] bundles = _bundleContext.getBundles();
    final List<ITargetLocation> bundleContainers = new ArrayList<ITargetLocation>();
    final Set<File> dirs = new HashSet<File>();
    for (final Bundle bundle : bundles) {
      {
        final EquinoxBundle bundleImpl = ((EquinoxBundle) bundle);
        Module _module = bundleImpl.getModule();
        ModuleRevision _currentRevision = _module.getCurrentRevision();
        Object _revisionInfo = _currentRevision.getRevisionInfo();
        final BundleInfo.Generation generation = ((BundleInfo.Generation) _revisionInfo);
        BundleFile _bundleFile = generation.getBundleFile();
        final File file = _bundleFile.getBaseFile();
        final File folder = file.getParentFile();
        boolean _contains = dirs.contains(folder);
        boolean _not = (!_contains);
        if (_not) {
          dirs.add(folder);
          String _absolutePath = folder.getAbsolutePath();
          ITargetLocation _newDirectoryLocation = tpService.newDirectoryLocation(_absolutePath);
          bundleContainers.add(_newDirectoryLocation);
        }
      }
    }
    final ITargetLocation[] bundleContainersArray = ((ITargetLocation[])Conversions.unwrapArray(bundleContainers, ITargetLocation.class));
    targetDef.setTargetLocations(bundleContainersArray);
    String _oSArch = Platform.getOSArch();
    targetDef.setArch(_oSArch);
    String _oS = Platform.getOS();
    targetDef.setOS(_oS);
    String _wS = Platform.getWS();
    targetDef.setWS(_wS);
    String _nL = Platform.getNL();
    targetDef.setNL(_nL);
    tpService.saveTargetDefinition(targetDef);
    final Job job = new LoadTargetDefinitionJob(targetDef);
    job.schedule();
    job.join();
  }
  
  /**
   * call a command the selection file, if file is null or empty it will call the command without selection
   */
  public static void invokeCommandOnSelectedFile(final String commandId, final String file) {
    try {
      final IWorkspace ws = ResourcesPlugin.getWorkspace();
      final IWorkbench wb = PlatformUI.getWorkbench();
      ICommandService _service = wb.<ICommandService>getService(ICommandService.class);
      final ICommandService commandService = ((ICommandService) _service);
      IHandlerService _service_1 = wb.<IHandlerService>getService(IHandlerService.class);
      final IHandlerService handlerService = ((IHandlerService) _service_1);
      final Command command = commandService.getCommand(commandId);
      final ExecutionEvent executionEvent = handlerService.createExecutionEvent(command, null);
      Object _applicationContext = executionEvent.getApplicationContext();
      final IEvaluationContext context = ((IEvaluationContext) _applicationContext);
      boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(file);
      boolean _not = (!_isNullOrEmpty);
      if (_not) {
        IWorkspaceRoot _root = ws.getRoot();
        Path _path = new Path(file);
        final IFile iFile = _root.getFile(_path);
        IEvaluationContext _parent = context.getParent();
        StructuredSelection _structuredSelection = new StructuredSelection(iFile);
        _parent.addVariable(ISources.ACTIVE_MENU_SELECTION_NAME, _structuredSelection);
      }
      command.executeWithChecks(executionEvent);
      IResourcesSetupUtil.reallyWaitForAutoBuild();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public ClassLoader createClassLoader(final IJavaProject project) {
    try {
      final String[] classPathEntries = JavaRuntime.computeDefaultRuntimeClassPath(project);
      final ArrayList<URL> urlList = new ArrayList<URL>();
      for (int i = 0; (i < classPathEntries.length); i++) {
        {
          final String entry = classPathEntries[i];
          final Path path = new Path(entry);
          File _file = path.toFile();
          URI _uRI = _file.toURI();
          final URL url = _uRI.toURL();
          urlList.add(url);
        }
      }
      Class<? extends IJavaProject> _class = project.getClass();
      final ClassLoader parentClassLoader = _class.getClassLoader();
      return new URLClassLoader(((URL[])Conversions.unwrapArray(urlList, URL.class)), parentClassLoader);
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public void waitFileExistOrAssert(final String fileName, final int retry, final long sleep) {
    final IWorkspace ws = ResourcesPlugin.getWorkspace();
    ExclusiveRange _doubleDotLessThan = new ExclusiveRange(0, retry, true);
    for (final Integer i : _doubleDotLessThan) {
      {
        IWorkspaceRoot _root = ws.getRoot();
        Path _path = new Path(fileName);
        IFile _file = _root.getFile(_path);
        boolean _exists = _file.exists();
        if (_exists) {
          return;
        }
        try {
          Thread.sleep(sleep);
        } catch (final Throwable _t) {
          if (_t instanceof InterruptedException) {
            final InterruptedException e = (InterruptedException)_t;
          } else {
            throw Exceptions.sneakyThrow(_t);
          }
        }
      }
    }
    this.assertFileExists(fileName);
  }
  
  /**
   * relaunch the  waitForJobs several times in case some other background task
   * also wait for idle time to triggers new jobs
   */
  public static void reallyWaitForJobs(final int retry) {
    try {
      IntegerRange _upTo = new IntegerRange(0, retry);
      for (final Integer i : _upTo) {
        {
          WorkspaceTestHelper.waitForJobs();
          Thread.sleep(100);
        }
      }
      WorkspaceTestHelper.waitForJobs();
    } catch (Throwable _e) {
      throw Exceptions.sneakyThrow(_e);
    }
  }
  
  public static void waitForJobs() {
    while ((!Job.getJobManager().isIdle())) {
      WorkspaceTestHelper.delay(100);
    }
  }
  
  private static boolean closed = false;
  
  public static void delay(final long waitTimeMillis) {
    final Display display = Display.getCurrent();
    IWorkbench _workbench = PlatformUI.getWorkbench();
    _workbench.addWindowListener(
      new IWindowListener() {
        @Override
        public void windowActivated(final IWorkbenchWindow window) {
        }
        
        @Override
        public void windowClosed(final IWorkbenchWindow window) {
          WorkspaceTestHelper.closed = true;
        }
        
        @Override
        public void windowDeactivated(final IWorkbenchWindow window) {
        }
        
        @Override
        public void windowOpened(final IWorkbenchWindow window) {
        }
      });
    boolean _notEquals = (!Objects.equal(display, null));
    if (_notEquals) {
      long _currentTimeMillis = System.currentTimeMillis();
      final long endTimeMillis = (_currentTimeMillis + waitTimeMillis);
      while (((System.currentTimeMillis() < endTimeMillis) && (!WorkspaceTestHelper.closed))) {
        boolean _readAndDispatch = display.readAndDispatch();
        boolean _not = (!_readAndDispatch);
        if (_not) {
          display.sleep();
        }
      }
      display.update();
    } else {
      try {
        Thread.sleep(waitTimeMillis);
      } catch (final Throwable _t) {
        if (_t instanceof InterruptedException) {
          final InterruptedException e = (InterruptedException)_t;
        } else {
          throw Exceptions.sneakyThrow(_t);
        }
      }
    }
  }
}
